<?php

namespace Drupal\monarch_entity_yml;

use Drupal\Core\Config\FileStorage;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Defines import actions for the install profile.
 */
class Import {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Create an Import helper class.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Import pre-parsed entity data as entities.
   *
   * @param array $data
   *   The pre-parsed entity data.
   */
  public function importData(array $data) {
    foreach ($data as $definition) {
      if (!isset($definition['storage']) || !is_string($definition['storage'])) {
        throw new \Exception('Storage not specified for entity entry.');
      }

      if (!isset($definition['values']) || !is_array($definition['values']) || empty($definition['values'])) {
        throw new \Exception('No valid values for entity entry.');
      }

      try {
        $entity = $this->entityTypeManager->getStorage($definition['storage'])->create($definition['values']);
        $entity->save();
      }
      catch (\Throwable $e) {
        if ($entity) {
          throw new \Exception('Error importing ' . $definition['storage'] . ':' . ($definition['values']['uuid'][0]['value'] ?? 'no_UUID') . ' - ' . $e->getMessage());
        }
        else {
          throw $e;
        }
      }
    }
  }

  /**
   * Generates baseline content (intended for after config-import).
   *
   * @param string $path
   *   The path to import files from.
   */
  public function import(string $path) {
    $storage = new FileStorage(
      $path
    );

    foreach ($storage->readMultiple($storage->listAll('')) as $data) {
      $this->importData($data);
    }
  }

}
